@@ -1,51 +1,38 @@
-/***************************************************************************
- *   Copyright © 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2017 Jan Grulich <jgrulich@redhat.com>                    *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "OdrsReviewsBackend.h"
 #include "AppStreamIntegration.h"
 #include "CachedNetworkAccessManager.h"
 
-#include <ReviewsBackend/Review.h>
 #include <ReviewsBackend/Rating.h>
+#include <ReviewsBackend/Review.h>
 
 #include <resources/AbstractResource.h>
 #include <resources/AbstractResourcesBackend.h>
 
 #include <KIO/FileCopyJob>
-#include <KUser>
 #include <KLocalizedString>
+#include <KUser>
 
+#include "libdiscover_debug.h"
 #include <QCryptographicHash>
 #include <QDir>
-#include "libdiscover_debug.h"
-#include <QJsonArray>
-#include <QJsonDocument>
-#include <QJsonObject>
 #include <QFile>
 #include <QFileInfo>
+#include <QJsonArray>
+#include <QJsonObject>
 #include <QNetworkAccessManager>
-#include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QStandardPaths>
 
+#include <QFutureWatcher>
+#include <QtConcurrentRun>
+
 // #define APIURL "http://127.0.0.1:5000/1.0/reviews/api"
 #define APIURL "https://odrs.gnome.org/1.0/reviews/api"
 
@@ -80,6 +67,11 @@ OdrsReviewsBackend::OdrsReviewsBackend()
     }
 }
 
+OdrsReviewsBackend::~OdrsReviewsBackend() noexcept
+{
+    qDeleteAll(m_ratings);
+}
+
 void OdrsReviewsBackend::ratingsFetched(KJob *job)
 {
     m_isFetching = false;
@@ -118,12 +110,12 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
     m_isFetching = true;
 
     const QJsonDocument document(QJsonObject{
-            {QStringLiteral("app_id"), app->appstreamId()},
-            {QStringLiteral("distro"), osName()},
-            {QStringLiteral("user_hash"), userHash()},
-            {QStringLiteral("version"), app->isInstalled() ? app->installedVersion() : app->availableVersion()},
-            {QStringLiteral("locale"), QLocale::system().name()},
-            {QStringLiteral("limit"), -1}
+        {QStringLiteral("app_id"), app->appstreamId()},
+        {QStringLiteral("distro"), osName()},
+        {QStringLiteral("user_hash"), userHash()},
+        {QStringLiteral("version"), app->isInstalled() ? app->installedVersion() : app->availableVersion()},
+        {QStringLiteral("locale"), QLocale::system().name()},
+        {QStringLiteral("limit"), -1},
     });
 
     const auto json = document.toJson(QJsonDocument::Compact);
@@ -139,27 +131,24 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
 
 void OdrsReviewsBackend::reviewsFetched()
 {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
     QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(reply);
     const QByteArray data = reply->readAll();
-#if (QT_VERSION < QT_VERSION_CHECK(5, 15, 0))
     const auto networkError = reply->error();
-#else
-    const auto networkError = reply->networkError();
-#endif
     if (networkError != QNetworkReply::NoError) {
         qCWarning(LIBDISCOVER_LOG) << "error fetching reviews:" << reply->errorString() << data;
+        Q_EMIT error(i18n("Error while fetching reviews: %1", reply->errorString()));
         m_isFetching = false;
         return;
     }
 
     const QJsonDocument document = QJsonDocument::fromJson(data);
-    AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
+    AbstractResource *resource = qobject_cast<AbstractResource *>(reply->request().originatingObject());
     Q_ASSERT(resource);
     parseReviews(document, resource);
 }
 
-Rating * OdrsReviewsBackend::ratingForApplication(AbstractResource *app) const
+Rating *OdrsReviewsBackend::ratingForApplication(AbstractResource *app) const
 {
     if (app->appstreamId().isEmpty()) {
         return nullptr;
@@ -171,11 +160,11 @@ Rating * OdrsReviewsBackend::ratingForApplication(AbstractResource *app) const
 void OdrsReviewsBackend::submitUsefulness(Review *review, bool useful)
 {
     const QJsonDocument document(QJsonObject{
-                     {QStringLiteral("app_id"), review->applicationName()},
-                     {QStringLiteral("user_skey"), review->getMetadata(QStringLiteral("ODRS::user_skey")).toString()},
-                     {QStringLiteral("user_hash"), userHash()},
-                     {QStringLiteral("distro"), osName()},
-                     {QStringLiteral("review_id"), QJsonValue(double(review->id()))} //if we really need uint64 we should get it in QJsonValue
+        {QStringLiteral("app_id"), review->applicationName()},
+        {QStringLiteral("user_skey"), review->getMetadata(QStringLiteral("ODRS::user_skey")).toString()},
+        {QStringLiteral("user_hash"), userHash()},
+        {QStringLiteral("distro"), osName()},
+        {QStringLiteral("review_id"), QJsonValue(double(review->id()))}, // if we really need uint64 we should get it in QJsonValue
     });
 
     QNetworkRequest request(QUrl(QStringLiteral(APIURL) + (useful ? QLatin1String("/upvote") : QLatin1String("/downvote"))));
@@ -188,16 +177,13 @@ void OdrsReviewsBackend::submitUsefulness(Review *review, bool useful)
 
 void OdrsReviewsBackend::usefulnessSubmitted()
 {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-#if (QT_VERSION < QT_VERSION_CHECK(5, 15, 0))
+    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
     const auto networkError = reply->error();
-#else
-    const auto networkError = reply->networkError();
-#endif
     if (networkError == QNetworkReply::NoError) {
         qCWarning(LIBDISCOVER_LOG) << "Usefulness submitted";
     } else {
         qCWarning(LIBDISCOVER_LOG) << "Failed to submit usefulness: " << reply->errorString();
+        Q_EMIT error(i18n("Error while submitting usefulness: %1", reply->errorString()));
     }
     reply->deleteLater();
 }
@@ -209,16 +195,19 @@ QString OdrsReviewsBackend::userName() const
 
 void OdrsReviewsBackend::submitReview(AbstractResource *res, const QString &summary, const QString &description, const QString &rating)
 {
-    QJsonObject map = {{QStringLiteral("app_id"), res->appstreamId()},
-                     {QStringLiteral("user_skey"), res->getMetadata(QStringLiteral("ODRS::user_skey")).toString()},
-                     {QStringLiteral("user_hash"), userHash()},
-                     {QStringLiteral("version"), res->isInstalled() ? res->installedVersion() : res->availableVersion()},
-                     {QStringLiteral("locale"), QLocale::system().name()},
-                     {QStringLiteral("distro"), osName()},
-                     {QStringLiteral("user_display"), QJsonValue::fromVariant(KUser().property(KUser::FullName))},
-                     {QStringLiteral("summary"), summary},
-                     {QStringLiteral("description"), description},
-                     {QStringLiteral("rating"), rating.toInt() * 10}};
+    Q_ASSERT(res);
+    QJsonObject map = {
+        {QStringLiteral("app_id"), res->appstreamId()},
+        {QStringLiteral("user_skey"), res->getMetadata(QStringLiteral("ODRS::user_skey")).toString()},
+        {QStringLiteral("user_hash"), userHash()},
+        {QStringLiteral("version"), res->isInstalled() ? res->installedVersion() : res->availableVersion()},
+        {QStringLiteral("locale"), QLocale::system().name()},
+        {QStringLiteral("distro"), osName()},
+        {QStringLiteral("user_display"), QJsonValue::fromVariant(KUser().property(KUser::FullName))},
+        {QStringLiteral("summary"), summary},
+        {QStringLiteral("description"), description},
+        {QStringLiteral("rating"), rating.toInt() * 10},
+    };
 
     const QJsonDocument document(map);
 
@@ -239,17 +228,19 @@ void OdrsReviewsBackend::submitReview(AbstractResource *res, const QString &summ
 
 void OdrsReviewsBackend::reviewSubmitted(QNetworkReply *reply)
 {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 15, 0))
     const auto networkError = reply->error();
-#else
-    const auto networkError = reply->networkError();
-#endif
     if (networkError == QNetworkReply::NoError) {
-        AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
+        AbstractResource *resource = qobject_cast<AbstractResource *>(reply->request().originatingObject());
+        Q_ASSERT(resource);
         qCWarning(LIBDISCOVER_LOG) << "Review submitted" << resource;
-        const QJsonDocument document({resource->getMetadata(QStringLiteral("ODRS::review_map")).toObject()});
-        parseReviews(document, resource);
+        if (resource) {
+            const QJsonDocument document({resource->getMetadata(QStringLiteral("ODRS::review_map")).toObject()});
+            parseReviews(document, resource);
+        } else {
+            qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: missing object";
+        }
     } else {
+        Q_EMIT error(i18n("Error while submitting review: %1", reply->errorString()));
         qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: " << reply->errorString();
     }
     reply->deleteLater();
@@ -257,30 +248,37 @@ void OdrsReviewsBackend::reviewSubmitted(QNetworkReply *reply)
 
 void OdrsReviewsBackend::parseRatings()
 {
-    QFile ratingsDocument(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/ratings/ratings"));
-    if (ratingsDocument.open(QIODevice::ReadOnly)) {
-        QJsonDocument jsonDocument = QJsonDocument::fromJson(ratingsDocument.readAll());
+    auto fw = new QFutureWatcher<QJsonDocument>(this);
+    connect(fw, &QFutureWatcher<QJsonDocument>::finished, this, [this, fw] {
+        const QJsonDocument jsonDocument = fw->result();
+        fw->deleteLater();
         const QJsonObject jsonObject = jsonDocument.object();
         m_ratings.reserve(jsonObject.size());
         for (auto it = jsonObject.begin(); it != jsonObject.end(); it++) {
             QJsonObject appJsonObject = it.value().toObject();
 
-            const int ratingCount =  appJsonObject.value(QLatin1String("total")).toInt();
-            QVariantMap ratingMap = { { QStringLiteral("star0"), appJsonObject.value(QLatin1String("star0")).toInt() },
-                                      { QStringLiteral("star1"), appJsonObject.value(QLatin1String("star1")).toInt() },
-                                      { QStringLiteral("star2"), appJsonObject.value(QLatin1String("star2")).toInt() },
-                                      { QStringLiteral("star3"), appJsonObject.value(QLatin1String("star3")).toInt() },
-                                      { QStringLiteral("star4"), appJsonObject.value(QLatin1String("star4")).toInt() },
-                                      { QStringLiteral("star5"), appJsonObject.value(QLatin1String("star5")).toInt() } };
+            const int ratingCount = appJsonObject.value(QLatin1String("total")).toInt();
+            int ratingMap[] = {
+                appJsonObject.value(QLatin1String("star0")).toInt(),
+                appJsonObject.value(QLatin1String("star1")).toInt(),
+                appJsonObject.value(QLatin1String("star2")).toInt(),
+                appJsonObject.value(QLatin1String("star3")).toInt(),
+                appJsonObject.value(QLatin1String("star4")).toInt(),
+                appJsonObject.value(QLatin1String("star5")).toInt(),
+            };
 
             Rating *rating = new Rating(it.key(), ratingCount, ratingMap);
-            rating->setParent(this);
             m_ratings.insert(it.key(), rating);
         }
-        ratingsDocument.close();
-
         Q_EMIT ratingsReady();
-    }
+    });
+    fw->setFuture(QtConcurrent::run([] {
+        QFile ratingsDocument(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/ratings/ratings"));
+        if (!ratingsDocument.open(QIODevice::ReadOnly)) {
+            return QJsonDocument::fromJson({});
+        }
+        return QJsonDocument::fromJson(ratingsDocument.readAll());
+    }));
 }
 
 void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractResource *resource)
@@ -301,11 +299,18 @@ void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractRes
                 const int usefulTotal = review.value(QStringLiteral("karma_down")).toInt() + usefulFavorable;
                 QDateTime dateTime;
                 dateTime.setSecsSinceEpoch(review.value(QStringLiteral("date_created")).toInt());
-                ReviewPtr r(new Review(review.value(QStringLiteral("app_id")).toString(), resource->packageName(),
-                                       review.value(QStringLiteral("locale")).toString(), review.value(QStringLiteral("summary")).toString(),
-                                       review.value(QStringLiteral("description")).toString(), review.value(QStringLiteral("user_display")).toString(),
-                                       dateTime, true, review.value(QStringLiteral("review_id")).toInt(),
-                                       review.value(QStringLiteral("rating")).toInt() / 10, usefulTotal, usefulFavorable,
+                ReviewPtr r(new Review(review.value(QStringLiteral("app_id")).toString(),
+                                       resource->packageName(),
+                                       review.value(QStringLiteral("locale")).toString(),
+                                       review.value(QStringLiteral("summary")).toString(),
+                                       review.value(QStringLiteral("description")).toString(),
+                                       review.value(QStringLiteral("user_display")).toString(),
+                                       dateTime,
+                                       true,
+                                       review.value(QStringLiteral("review_id")).toInt(),
+                                       review.value(QStringLiteral("rating")).toInt() / 10,
+                                       usefulTotal,
+                                       usefulFavorable,
                                        review.value(QStringLiteral("version")).toString()));
                 // We can also receive just a json with app name and user info so filter these out as there is no review
                 if (!r->summary().isEmpty() && !r->reviewText().isEmpty()) {
@@ -323,22 +328,22 @@ void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractRes
     }
 }
 
-bool OdrsReviewsBackend::isResourceSupported(AbstractResource* res) const
+bool OdrsReviewsBackend::isResourceSupported(AbstractResource *res) const
 {
     return !res->appstreamId().isEmpty();
 }
 
-void OdrsReviewsBackend::emitRatingFetched(AbstractResourcesBackend* b, const QList<AbstractResource *>& resources) const
+void OdrsReviewsBackend::emitRatingFetched(AbstractResourcesBackend *b, const QList<AbstractResource *> &resources) const
 {
     b->emitRatingsReady();
-    foreach(AbstractResource* res, resources) {
+    foreach (AbstractResource *res, resources) {
         if (m_ratings.contains(res->appstreamId())) {
             Q_EMIT res->ratingFetched();
         }
     }
 }
 
-QNetworkAccessManager * OdrsReviewsBackend::nam()
+QNetworkAccessManager *OdrsReviewsBackend::nam()
 {
     if (!m_delayedNam) {
         m_delayedNam = new CachedNetworkAccessManager(QStringLiteral("odrs"), this);
